
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _YOINKAPP_HH_
#define _YOINKAPP_HH_

/**
 * @file YoinkApp.hh
 * This is the big enchilada.
 */

#include <iostream>
#include <string>

#include <Moof/Camera.hh>
#include <Moof/Dispatcher.hh>
#include <Moof/Engine.hh>
#include <Moof/Interpolator.hh>
#include <Moof/Math.hh>
#include <Moof/Scene.hh>

#include "Character.hh"
#include "TilemapFont.hh"


class YoinkApp : public Mf::Engine
{
public:
	YoinkApp(int argc, char* argv[]);
	~YoinkApp();

private:
	void update(Mf::Scalar t, Mf::Scalar dt);
	void draw(Mf::Scalar alpha);
	void handleEvent(const Mf::Event& event);

	/**
	 * Set OpenGL to a state we can know and depend on.
	 */
	void setupGL();
	void contextRecreated(const Mf::Notification& note);

	Character* someChar;
	TilemapFont *font;

	Mf::Cerpv2 interp;
	Mf::Lerps fadeIn;

	Mf::Camera camera;
	Mf::Scene* testScene;

	Mf::Scalar x, y, z;

	Mf::Scalar state;
	Mf::Scalar prevstate;
};


#endif // _YOINKAPP_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

